import os
import argparse
from abyz.compiler import compile

def main():

    # Parse in and out arguments
    parser = argparse.ArgumentParser(description ='abyz script compiler')
    parser.add_argument('-c', '--code', type = str, help ='abyz code to be compiled', required=True)
    parser.add_argument('-o', '--output', type = str, help ='output file name')
    parser.add_argument('-x', '--exec', action='store_true', help ='execute the program')

    args = parser.parse_args()
    code = args.code
    output = args.output
    exec_code = args.exec

    out = compile(code, output)

    if exec_code:
        os.system("python "+out)

if __name__ == "__main__":
    
    main()


# Some selected keywords
selected = ["return","def"]
control_struct = ["if","else","for","while"] # no need to include elif, already accounted in if

def correct(content: list[str]) -> list[str]:
    """Function to perform some corrections with semicolons and parentheses"""

    compiled = []
    for row in content:

        # Add extra space in defs and returns (not needed for now)
        """
        for word in selected:
            if word in row:
                row = row.replace(word,word+" ")
        """
        
        # Add ) at the end of the line when prints
        if "print(" in row:
            row += ")"

        # Add : at the end of the line in control structures
        for word in control_struct:
            if word in row:
                row += ":"

        # Add ) at the end of the line when prints
        if "def" in row:
            row += ":"

        compiled.append(row)

    return compiled


import argparse
from .replacements import replace_singles
from .correct import correct

def compile(code: str, output: str | None = None) -> str:
    """Main compiler function, which takes abyz code and transpiles it to python code"""

    if output is None:
        output = code.replace(".abyz",".py")

    # Read abyz code to compile
    with open(code, encoding='utf-8') as file:
        content = file.read()

    content = content.split("\n")

    # Replace reserved words with their respective python code
    content = replace_singles(content,"numbers")
    content = replace_singles(content,"symbols")

    # Remove spaces, commented for now due to some issues
    #for i, row in enumerate(content):
    #    content[i] = row.replace(" ", "")

    # Add extra corrections for some spaces
    content = correct(content)

    # Write compiled code to executable file
    with open(output, "w") as f:
        f.write("\n".join(content) + "\n")

    # Return output file path for being executed if needed
    return output

if __name__=="__main__":

    # Parse in and out arguments
    parser = argparse.ArgumentParser(description ='abyz script compiler')
    parser.add_argument('-c', '--code', type = str, help ='abyz code to be compiled', required=True)
    parser.add_argument('-o', '--output', type = str, help ='output file name')

    args = parser.parse_args()
    code = args.code
    output = args.output

    compile(code, output)


import json
from .global_constants import NUMBERS_PATH, SINGLES_PATH

with open(NUMBERS_PATH) as f:
    numbers = json.load(f)

with open(SINGLES_PATH) as f:
    singles = json.load(f)

def find_all_occurrences(string: str, substring: str) -> list[int]:
    """Function to find all occurrences of a substring in a string (not used right now)"""

    indices = []
    start = 0
    while start < len(string):
        # Find the next occurrence of the substring
        start = string.find(substring, start)
        if start == -1:  # No more occurrences
            break
        indices.append(start)
        start += len(substring)  # Move past the current occurrence
    return indices

def replace_singles(content: list[str], type: str) -> list[str]:
    """Function to replace single elements, such as symbols and numbers"""

    if type=="symbols":
        reserved = singles
    elif type=="numbers":
        reserved = numbers
    
    compiled = []
    for row in content:
        for word in reserved:
            if word in row:
                row = row.replace(word, str(reserved[word]))
        compiled.append(row)
    return compiled



import sys
import pathlib

ABYZ_SRC_PATH = str(pathlib.Path(__file__).parent.resolve())

if sys.platform=="win32":
    SINGLES_PATH = ABYZ_SRC_PATH + r"\reserved\single_reserved.json"
    NUMBERS_PATH = ABYZ_SRC_PATH + r"\reserved\numbers.json"
else:
    SINGLES_PATH = ABYZ_SRC_PATH + "/reserved/single_reserved.json"
    NUMBERS_PATH = ABYZ_SRC_PATH + "/reserved/numbers.json"
